<!DOCTYPE html>
<html>
{% include 'head.html' %}
<body>
    <div id="centered-container">
        {% include 'header.html' %}
        <div id="start-text">
<p>
This page does not exist (404) :/
</p>

    </div>
        {% include 'footer.html' %}
    </div>
</body>
</html>
