<div id="footer-bar">
    v{{ version}}
    // Copyright 2023-2025 <a href="https://cogsci.nl">CogSci</a>
    // <a href="/terms">Terms and conditions</a>
    // <a href='https://github.com/open-cogsci/sigmund-ai'>Open source GPL3</a>
</div>
